/**
 * Module imports
 */
var _       = require( 'underscore' )._;
var assert  = require( 'assert' );

/**
 * Module exports
 */
exports.header  = header;
exports.request = request;
exports.status  = status;

/**
 * Arbitary header test macro
 *
 * @api public
 * @type {Function}
 * @param {String} name of the header to check for
 * @param {String} value to test against
 * @return {Function}
 */
function header ( name, value ) {
  return function ( err, res ) {
    assert.ifError( err );
    assert.equal( res.headers[name], value );
  }
}

/**
 * statusCode test macro
 *
 * @api public
 * @type {Function}
 * @param {Number} code the statusCode should match
 * @return {Function}
 */
function status ( code ) {
  return function ( err, res ) {
    assert.ifError( err );
    assert.equal( res.statusCode, code );
  }
}

/**
 * Request/Response cycyle macro
 *
 * @api public
 * @type {Function}
 * @param {Object} request Options for the http request
 * @param {Object} response Criteria the response should match
 * @param {Object} vows Arbitary vows to be executed
 * @return {Function}
 */
function request ( request, response, vows ) {
  var context;
  
  context = {
    topic: function ( client ) {
      var nameSplit = this.context.name.split(/ +/);
      var method    = nameSplit[0].toLowerCase()
      var path      = nameSplit[1]

      request = _.extend({
        method: method,
        path: path
      }, request );

      return client[( method === 'delete' ) ? 'del' : method]( request );
    }
  };

  if ( response.status ) {
    context['should respond with a ' + response.status] = status( response.status );
  }

  if ( response.headers ) {
    _.each( response.headers, function ( value, key ) {
      context['should respond with ' + key] = header ( key, value );
    });
  }
  if ( vows ) {
    _.each( vows, function ( vow, name ) {
      context[name] = vow;
    })
  }

  return context;
}

